@@ -14,6 +14,7 @@ in the source distribution for its full text.
 
 #include "String.h"
 
+#include "config.h"
 #include "debug.h"
 
 #define ColorPair(i,j) COLOR_PAIR((7-i)*8+j)
@@ -93,6 +94,9 @@ typedef enum ColorElements_ {
    CPU_NORMAL,
    CPU_KERNEL,
    HELP_BOLD,
+   CPU_IOWAIT,
+   CPU_IRQ,
+   CPU_SOFTIRQ,
    LAST_COLORELEMENT
 } ColorElements;
 
@@ -160,6 +164,7 @@ void CRT_done() {
 int CRT_readKey() {
    nocbreak();
    cbreak();
+   nodelay(stdscr, FALSE);
    int ret = getch();
    halfdelay(CRT_delay);
    return ret;
@@ -177,7 +182,7 @@ void CRT_enableDelay() {
 
 void CRT_handleSIGSEGV(int signal) {
    CRT_done();
-   fprintf(stderr, "Aborted. Please report bug at http://htop.sf.net");
+   fprintf(stderr, "htop " VERSION " aborted. Please report bug at http://htop.sf.net\n");
    exit(1);
 }
 
@@ -250,6 +255,9 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = A_BOLD;
       CRT_colors[CHECK_MARK] = A_NORMAL;
       CRT_colors[CHECK_TEXT] = A_NORMAL;
+      CRT_colors[CPU_IOWAIT] = A_NORMAL;
+      CRT_colors[CPU_IRQ] = A_BOLD;
+      CRT_colors[CPU_SOFTIRQ] = A_BOLD;
    } else if (CRT_colorScheme == COLORSCHEME_BLACKONWHITE) {
       CRT_colors[RESET_COLOR] = ColorPair(Black,White);
       CRT_colors[DEFAULT_COLOR] = ColorPair(Black,White);
@@ -302,6 +310,9 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = ColorPair(Blue,White);
       CRT_colors[CHECK_MARK] = ColorPair(Black,White);
       CRT_colors[CHECK_TEXT] = ColorPair(Black,White);
+      CRT_colors[CPU_IOWAIT] = A_BOLD | ColorPair(Black, Black);
+      CRT_colors[CPU_IRQ] = ColorPair(Blue,White);
+      CRT_colors[CPU_SOFTIRQ] = ColorPair(Blue,White);
    } else if (CRT_colorScheme == COLORSCHEME_BLACKONWHITE2) {
       CRT_colors[RESET_COLOR] = ColorPair(Black,Black);
       CRT_colors[DEFAULT_COLOR] = ColorPair(Black,Black);
@@ -354,6 +365,9 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = ColorPair(Blue,Black);
       CRT_colors[CHECK_MARK] = ColorPair(Black,Black);
       CRT_colors[CHECK_TEXT] = ColorPair(Black,Black);
+      CRT_colors[CPU_IOWAIT] = A_BOLD | ColorPair(Black, Black);
+      CRT_colors[CPU_IRQ] = A_BOLD | ColorPair(Blue,Black);
+      CRT_colors[CPU_SOFTIRQ] = ColorPair(Blue,Black);
    } else if (CRT_colorScheme == COLORSCHEME_MIDNIGHT) {
       CRT_colors[RESET_COLOR] = ColorPair(White,Blue);
       CRT_colors[DEFAULT_COLOR] = ColorPair(White,Blue);
@@ -406,6 +420,9 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = ColorPair(Cyan,Blue);
       CRT_colors[CHECK_MARK] = A_BOLD | ColorPair(White,Blue);
       CRT_colors[CHECK_TEXT] = A_NORMAL | ColorPair(White,Blue);
+      CRT_colors[CPU_IOWAIT] = ColorPair(Yellow,Blue);
+      CRT_colors[CPU_IRQ] = A_BOLD | ColorPair(Black,Blue);
+      CRT_colors[CPU_SOFTIRQ] = ColorPair(Black,Blue);
    } else if (CRT_colorScheme == COLORSCHEME_BLACKNIGHT) {
       CRT_colors[RESET_COLOR] = ColorPair(Cyan,Black);
       CRT_colors[DEFAULT_COLOR] = ColorPair(Cyan,Black);
@@ -458,6 +475,9 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = ColorPair(Green,Black);
       CRT_colors[CHECK_MARK] = A_BOLD | ColorPair(Green,Black);
       CRT_colors[CHECK_TEXT] = ColorPair(Cyan,Black);
+      CRT_colors[CPU_IOWAIT] = ColorPair(Yellow,Black);
+      CRT_colors[CPU_IRQ] = A_BOLD | ColorPair(Blue,Black);
+      CRT_colors[CPU_SOFTIRQ] = ColorPair(Blue,Black);
    } else {
       /* Default */
       CRT_colors[RESET_COLOR] = ColorPair(White,Black);
@@ -511,5 +531,8 @@ void CRT_setColors(int colorScheme) {
       CRT_colors[CHECK_BOX] = ColorPair(Cyan,Black);
       CRT_colors[CHECK_MARK] = A_BOLD;
       CRT_colors[CHECK_TEXT] = A_NORMAL;
+      CRT_colors[CPU_IOWAIT] = A_BOLD | ColorPair(Black, Black);
+      CRT_colors[CPU_IRQ] = ColorPair(Yellow,Black);
+      CRT_colors[CPU_SOFTIRQ] = ColorPair(Magenta,Black);
    }
 }
