@@ -1,6 +1,6 @@
 /*
 htop
-(C) 2004 Hisham H. Muhammad
+(C) 2004-2006 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -16,22 +16,26 @@ in the source distribution for its full text.
 typedef struct CheckItem_ {
    Object super;
    char* text;
-   bool* value;
+   bool value;
+   bool* ref;
 } CheckItem;
 
-extern char* CHECKITEM_CLASS;
 }*/
 
-/* private property */
+#ifdef DEBUG
 char* CHECKITEM_CLASS = "CheckItem";
+#else
+#define CHECKITEM_CLASS NULL
+#endif
 
-CheckItem* CheckItem_new(char* text, bool* value) {
+CheckItem* CheckItem_new(char* text, bool* ref, bool value) {
    CheckItem* this = malloc(sizeof(CheckItem));
-   ((Object*)this)->class = CHECKITEM_CLASS;
+   Object_setClass(this, CHECKITEM_CLASS);
    ((Object*)this)->display = CheckItem_display;
    ((Object*)this)->delete = CheckItem_delete;
    this->text = text;
    this->value = value;
+   this->ref = ref;
    return this;
 }
 
@@ -43,11 +47,25 @@ void CheckItem_delete(Object* cast) {
    free(this);
 }
 
+void CheckItem_set(CheckItem* this, bool value) {
+   if (this->ref) 
+      *(this->ref) = value;
+   else
+      this->value = value;
+}
+
+bool CheckItem_get(CheckItem* this) {
+   if (this->ref) 
+      return *(this->ref);
+   else
+      return this->value;
+}
+
 void CheckItem_display(Object* cast, RichString* out) {
    CheckItem* this = (CheckItem*)cast;
    assert (this != NULL);
    RichString_write(out, CRT_colors[CHECK_BOX], "[");
-   if (*(this->value))
+   if (CheckItem_get(this))
       RichString_append(out, CRT_colors[CHECK_MARK], "x");
    else
       RichString_append(out, CRT_colors[CHECK_MARK], " ");
